from datetime import datetime, timedelta

class DateUtils:
    @staticmethod
    def float_to_datetime(timestamp: float) -> datetime:
        """
        将浮点数转换为日期时间对象

        :param timestamp: 浮点数，表示自 Unix 纪元以来的秒数
        :return: 日期时间对象
        """
        return datetime(1970, 1, 1) + timedelta(seconds=timestamp)

if __name__ == '__main__':
    # 示例用法
    _timestamp = 1672531199.0  # 示例浮点数
    date_time = DateUtils.float_to_datetime(_timestamp)
    print(date_time)



import logging
import logging.config
import os
from typing import AnyStr

from mag_tools.model.log_type import LogType
from mag_tools.utils.common.string_utils import StringUtils


class Logger:
    __instance = None

    def __init__(self, app: os.PathLike[AnyStr], root_dir: str):
        """
        初始化 Logger 实例，配置日志记录器和日志文件路径。

        :param app: 应用程序路径
        :param root_dir: 根目录路径
        """
        app_name = os.path.splitext(os.path.basename(app))[0]
        self.__root_dir = root_dir

        log_dir = os.path.join(self.__root_dir, "data", app_name, "logs")
        os.makedirs(log_dir, exist_ok=True)

        logging.config.fileConfig(f"{self.__root_dir}\\data\\logging.conf", defaults={'logdir': str(log_dir)})

        self.root_logger = logging.getLogger()

        self.frame_logger = logging.getLogger('frame')

        self.service_logger = logging.getLogger('service')

        self.performance_logger = logging.getLogger('performance')

        Logger.__instance = self

    @staticmethod
    def initialize(app: os.PathLike[AnyStr], root_dir: str):
        """
        初始化 Logger 单例实例。

        :param app: 应用程序路径
        :param root_dir: 根目录路径
        """
        if Logger.__instance is None:
            Logger(app, root_dir)

    @staticmethod
    def debug(*args):
        """
        记录调试级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = args[0]
            Logger.__instance.__debug(LogType.FRAME, message)
        elif len(args) == 2:
            log_type, message = args
            Logger.__instance.__debug(log_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @staticmethod
    def info(*args):
        """
        记录信息级别的日志信息。

        :param args: 日志信息参数，可以是单个消息、(logger_type, message) 或 (logger_type, message, is_highlight) 元组
        """
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = args[0]
            Logger.__instance.__info(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            Logger.__instance.__info(logger_type, message)
        elif len(args) == 3:
            logger_type, message, is_highlight = args
            if is_highlight:
                Logger.__instance.__info(logger_type, '*'*(StringUtils.get_print_width(message)+8))
                Logger.__instance.__info(logger_type, f'*** {message} ***')
                Logger.__instance.__info(logger_type, '*'*(StringUtils.get_print_width(message)+8))
            else:
                Logger.__instance.__info(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @staticmethod
    def warning(*args):
        """
        记录警告级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = args[0]
            Logger.__instance.__warning(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            Logger.__instance.__warning(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    @staticmethod
    def error(*args):
        """
        记录错误级别的日志信息。

        :param args: 日志信息参数，可以是单个消息或 (logger_type, message) 元组
        """
        if Logger.__instance is None:
            raise ValueError("Logger not initialized. Please call initialize first.")

        if len(args) == 1:
            message = str(args[0]) if isinstance(args[0], Exception) else args[0]
            Logger.__instance.__error(LogType.FRAME, message)
        elif len(args) == 2:
            logger_type, message = args
            message = str(message) if isinstance(message, Exception) else message
            Logger.__instance.__error(logger_type, message)
        else:
            raise ValueError("Invalid number of arguments")

    def __debug(self, logger_type: LogType, message: str):
        """
        内部方法，记录调试级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.debug(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.debug(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.debug(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.debug(message)

    def __info(self, logger_type: LogType, message: str):
        """
        内部方法，记录信息级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.info(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.info(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.info(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.info(message)

    def __warning(self, logger_type: LogType, message: str):
        """
        内部方法，记录警告级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.warning(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.warning(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.warning(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.warning(message)

    def __error(self, logger_type: LogType, message: str):
        """
        内部方法，记录错误级别的日志信息。

        :param logger_type: 日志类型
        :param message: 日志消息
        """
        self.root_logger.error(message)

        if logger_type == LogType.FRAME:
            self.frame_logger.error(message)
        elif logger_type == LogType.SERVICE:
            self.service_logger.error(message)
        elif logger_type == LogType.PERFORMANCE:
            self.performance_logger.error(message)


import unittest
import pandas as pd
from mag_tools.utils.file.excel_utils import ExcelUtils


class TestExcelUtils(unittest.TestCase):

    def setUp(self):
        self.df = pd.DataFrame({
            'A': [None, None, None, None],
            'B': [None, 2, None, 4],
            'C': [None, None, None, None],
            'D': ['apple', 'banana', 'cherry', 'date']
        })

    def test_clear_empty(self):
        df = ExcelUtils.clear_empty(self.df)
        expected_output = pd.DataFrame({
            'B': [None, 2, None, 4],
            'D': ['apple', 'banana', 'cherry', 'date']
        }).reset_index(drop=True)
        pd.testing.assert_frame_equal(df, expected_output)

    def test_get_cells_of_row_by_keyword(self):
        result = ExcelUtils.get_cells_of_row_by_keyword(self.df, 3, 'a')
        expected_output = pd.Series(['date'], index=['D'])
        expected_output.name = None  # 确保预期输出没有 name 属性
        pd.testing.assert_series_equal(result, expected_output)

    def test_get_first_cell_of_row_by_keyword(self):
        result = ExcelUtils.get_first_cell_of_row_by_keyword(self.df, 3, 'a')
        self.assertEqual(result, 'date')

    def test_delete_row_with_keyword(self):
        df = ExcelUtils.delete_row_with_keyword(self.df, 3, 'a')
        expected_output = pd.DataFrame({
            'A': [None, None, None],
            'B': [None, 2, None],
            'C': [None, None, None],
            'D': ['apple', 'banana', 'cherry']
        }).reset_index(drop=True)
        pd.testing.assert_frame_equal(df, expected_output)

    def test_get_value_from_group(self):
        group = self.df.groupby('D').get_group('banana')
        result = ExcelUtils.get_value_from_group(group, 'B', default_value='default')
        self.assertEqual(2, result)

if __name__ == '__main__':
    unittest.main()
